'************************************************************************************
' SyncMM_V42.bas             MultiMasterSlave                              22.12.15 *
'************************************************************************************
$regfile = "m8def.dat"
$crystal = 8000000                                ' interner RC-Osz. wird verwendet
'$baud = 38400                                     ' DEBUG only

'$sim
Debug Off                                         '
'Osccal = &HB1                                     ' int. RC-Osz. Kalibrierung (nur bei Debug ON)

$framesize = 48                                   '
$swstack = 48                                     '
$hwstack = 48                                     '

Config Portd.2 = Input                            ' Input vom PCF INT-Pin
Portd.2 = 1                                       ' pull up ein
'-------------- Slave Adressen ----------
Const Pcf0_adr = &H40                             ' PCF8574_0 Basisadresse (64)(fr Sync-Erkennung)
Const Vmx_adr = &H62                              ' VideoMatrix Basisadresse (98)
Const Amx_adr = &H64                              ' AudioMatrix Basisadresse (100)
Const Osd_adr = &H60                              ' OSD Adresse (96)

'**************************************************************************************************
'***************************** TWI setup Tomtom library *******************************************
'**************************************************************************************************
Config Twi = 65000                                ' 65 kHz NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN
$include "tomtwi.bas"                             ' im selben Verzeichnis  NNNNNNNNNNNNNNNNNNNNNNNNN
Twcr = &B00000100                                 ' TWI gleich einschalten (nur TWEN setzen)
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
On Twi Twi_mst_slv_isr , Nosave                   ' ISR f. TWI    +++++++ DAS MUSS DRIN SEIN +++++++++
'****************************************************************************************************

'***************************** Setup als Slave ******************************************************
Twi_slv_addr = &H68                               ' eigene Slave Adresse (104)
Dim Twi_sr_buff(20) As Byte                       ' fr empfangenes byte (wg. EEProm-Daten fr Sync-Ausw.
Dim Twi_st_buff(5) As Byte                        ' fr zu sendende bytes (des slave)
Twi_sr_data = Varptr(twi_sr_buff(1))              ' Datenpuffer Empfang als slave
Twi_st_data = Varptr(twi_st_buff(1))              ' Datenbuffer Senden als slave
Twar = Twi_slv_addr                               ' I2C Adresse OHNE GCA
Gosub Twi_sense                                   ' Aktivierung der slave Funktion

'***************************** Steuerung der TX PTT/AGC ******************************************
Const Pcatx = 226                                 ' Adresse der FRAMs fr TX (hE2)
Const Pcarx = 224                                 ' Adresse der FRAMs fr RX (hE0)
Const Framaddr_tx = 160                           ' FRAM fr TX
Const Framtx_ptt = 104                            ' = 100 + 4  (4.byte (fr PTT))

Dim Pcaschalter As Byte                           ' Adresse des einzelnen TX FRAM ( = 2^(TXnum-1))
Dim Masterbyte(5) As Byte                         ' array der als Slave empfangenen bytes

'***************************** PCF8574, A/V MUX und TX Steuerung ***************************
Const Maske_14 = &B1001_0000                      ' RX1 und RX4 werden selektiert

Dim Pcf0port As Byte                              ' Status des Ports (= Sync Muster der 8 RX)
Dim Syncmuster_14 As Byte                         ' = NOT Pcfport AND Maske_12 (RX1 und 2)
Dim Syncmuster_14_old As Byte                     ' zum Erkennen von nderungen
Dim Sync_neu As Bit                               ' flag fr Sync-Zwangsauswertung

Dim Twi_pcf0_buff As Byte                         ' Buffer fr Pcf0 byte (lesen)

Dim Twi_vmx_buff(3) As Byte                       ' ..fr Vmux bytes (Vein und Vaus)
Dim Twi_txset_buff(2) As Byte                     ' ..fr Txset Steuerung (TXnum/Modus/Schalter)
Dim Twi_osd_buff(45) As Byte                      ' ..fr OSD Text

Dim Sync_event As Byte                            ' als Flag fr Sync-nderung
Dim Sync_aktiv As Byte                            ' flag fr Aktivierung / Deaktivierung der Auswertung

Dim Fehler_flag As Byte                           ' zum Loop-Abbruch und Neustart
Dim Schleifen As Byte                             ' fr Schleifenzhler

'************************** OSD Ausgabe **********************************
Dim Osdtext As String * 30                        ' Text fr OSD
Dim Osdstring As String * 30                      ' = Osdtext + "*"  fr Meldungen
Dim Osd_event As Bit                              ' flag fr OSD Aktivitt
Dim Osd_aktiv As Bit                              ' flag fr Aktivierung/Deakt. der OSD-Befehle
'************************** Timer0 ************************************
Config Timer0 = Timer , Prescale = 1024           ' 32,xxx ms bis berlauf
On Timer0 Isr_tim0
Dim Tick_count As Byte                            ' als ISR Zhler
Dim Tick_count_old As Byte                        ' zum Vergleich
Dim Osd_dauer As Word                             ' timing fr OSD Texte
Const Maxosd = 2                                  ' [s]     war 4
Const Reload = 195                                ' 195 ticks; ergibt 25ms

'********************** Temperatursensoren LM75 ***************************
Const Lm75_slave_addr = 145                       ' LM75-Basisadresse &H91 (read)
Dim Lm75_slave(2) As Byte                         ' falls MSB = 1 dann negative temp
Dim Temperatur(8)as Byte                          ' Array fr nur positive und ganzzahlige Temperatur
Dim Temp_aktiv As Bit                             ' flag zum Temperatur-Auslesen

'*********************************************************************************************
'************************* Sub declarations **************************************************
'*********************************************************************************************
Declare Sub Vmux(byval Vein As Byte , Byval Vaus As Byte)       ' kein Schalter bei Vmux
Declare Sub Txset(byval Txnum As Byte , Byval Modus As Byte , Byval Schalter As Byte)       ' Modus: 1 = PTT (2 = AGC)
Declare Sub Osd(byval Rxnum As Byte , Byval Osdstring As String , Byval Befehl As Byte)
Declare Sub Lm75(byval Rtxnum As Byte , Byval Lm75num As Byte , Byval Tnum As Byte)
                     ' Rtxnum: 0 = I2C-Bus, 1-8 = RXnum bzw. TXnum
                                            ' Lm75num: 0 = FRAM TX, 1 = FRAM RX, 2-7 = LM75 I2C Adresse
                                                                    ' Tnum (1-8)fr Ergebnis-Array
'*********************************************************************************************
Dim N As Byte                                     ' Zhlvariable
Dim Sync_ein(16) As Byte                          'Array fr Syncauswertung (VEin)

Enable Timer0                                     '
Enable Interrupts

$eeprom                                           ' hier oben ist's bequemer
Data 0 , 1 , 1 , 1 , 1 , 2 , 2 , 2 , 2 , 3 , 3 , 3 , 3 , 4 , 4 , 4 , 4
$data

'*************************************************************************
'***************************** Initialisierung ***************************
'*************************************************************************

Schleifen = 3                                     ' ...bis zur Fehlermeldung
Sync_event = 1                                    '  beim Start Zwangsauswertung
Sync_aktiv = 1                                    ' Auswertung aktiv
'Sync_neu = 1                                      ' auf jeden Fall neu setzen
Syncmuster_14_old = &B1111_1111                   ' damit beim ersten Mal sync IMMER ausgewertet wird NNNNNNNNNN

Debug "EEProm>Sync_ein"                           ' DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
Debug " "
For N = 1 To 16                                   ' Syncauswertungsdaten aus EEProm laden
   Readeeprom Sync_ein(n) , N
'   Debug Sync_ein(n)
Next N

Load Timer0 , Reload                              ' Timer startet (bei 256 - Reload)!!!
Debug "Auf geht's"
Debug " "
'***************************************************************************
'********************** MAIN LOOP ******************************************
'***************************************************************************
Wait 5                                            ' nach einem Reset mssen zuerst alle in Gang kommen NNNNNNNNNNNNNNNNNNNNNNNNNN
                                                  ' 2s reichen NICHT, 4 auch nicht
Do
   Twi_st_buff(2) = Sync_aktiv                    ' jetzt hier auch
'------ slave auswerten ------
   If Twi_slv_flag <> 0 Then                      ' ist beim slave was passiert?
      Debug "Slave event"
      Gosub Slave_rxtx                            ' ja, dann auswerten
   End If

   '------ Syncstatus neu abfragen ------
   If Pind.2 = 0 Then Sync_event = 1              ' POLLING des INT-Pins vom PCF
   If Sync_event = 1 And Sync_aktiv = 1 Then      ' PCF gendert UND Auswertung aktiv
      Debug ">> PCF auslesen"
      Sync_event = 0                              ' flag zurcksetzen
      Gosub Pcf_lesen                             ' PCF auslesen   (Zeile 480)
      If Sync_neu = 1 Then                        '
         Sync_neu = 0                             ' Flag rcksetzen
         Gosub Avmatrix                           ' Audio/Videomatrixwerte aus Syncmuster bestimmen
      End If
   End If

'------------- OSD Befehle weitergeben ---------------
   If Osd_aktiv = 1 And Osd_event = 1 Then
      Osdstring = "*" + Osdtext + "*"
      Call Osd(1 , Osdstring , 4)                 ' Lauftext mit Startbefehl fr RX1 (Lauftext???)
      Call Osd(4 , Osdstring , 4)                 ' ... und fr RX4
      Osd_event = 0                               ' flag zurcksetzen
      Osd_dauer = 0                               ' jetzt Zeit hochzhlen
   End If

'**************************** Tick_count related **********************************************
   If Tick_count <> Tick_count_old Then           ' nur abfragen, wenn Zhlerstand gendert

      '--------- OSD lschen --------------------
      If Osd_aktiv = 1 And Osd_dauer >= Maxosd Then       ' OSD Einblendung wieder aus
         Call Osd(1 , "" , 128)                   ' OSD reset Befehl RX1
         Call Osd(4 , "" , 128)                   ' ... und RX4
         Osd_aktiv = 0
      End If
      Tick_count_old = Tick_count
   End If                                         ' Tick_count Bedingung Ende
'***************************************************************************************************
'------------- Temperatur messen (DB0QI) -----------
   If Temp_aktiv = 1 Then                         ' Extra Befehl (sync T) lst das hier aus
      Call Lm75(1 , 0 , 1)                        ' LM75_0 an TX1 = T(1)
      Call Lm75(3 , 0 , 2)                        ' LM75_0 an TX3 = T(2)
      Call Lm75(4 , 0 , 3)                        ' LM75_0 an TX4 = T(3)
      Temp_aktiv = 0
      Osdtext = "TX1:" + Str(temperatur(1)) + "  TX3:" + Str(temperatur(2)) + "  TX4:" + Str(temperatur(3))
      Osdstring = "*" + Osdtext + " Grad" + "*"   ' fr Meldung
      Call Osd(1 , Osdstring , 4)                 ' Meldung auf RX1 mit Startbefehl
      Osd_aktiv = 1
      Osd_dauer = 0                               ' jetzt Zeit hochzhlen
   End If
Loop

End

'******************************************************************************************
'*************************** ISR / Subs etc. **********************************************
'******************************************************************************************
Isr_tim0:                                         ' Timer0 ISR
   Incr Tick_count                                ' ...alle 25ms
   Load Timer0 , Reload                           '... und reload
   If Tick_count = 40 Then                        ' 40*25 = 1s
      Tick_count = 0                              ' ...von vorne zhlen
      Incr Osd_dauer                              '
   End If
Return


'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'+++++++++++++++++++++++++++ MMSlave-Funktion RX/TX byte +++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Slave_rxtx:
Dim Temp As Byte                                  '
Dim Tempb As Byte
Dim Z As Byte

   Select Case Twi_slv_flag
      Case &H60                                   ' .... es ist was empfangen worden
         For Temp = 1 To Twi_sr_cntr
            Masterbyte(temp) = Twi_sr_buff(temp)  ' array fllen
         Next Temp

      Select Case Masterbyte(1)
         Case 47                                  ' NEU
            Debug "Case 47"
            For N = 2 To 17
               Sync_ein(n) = Masterbyte(n)
               Debug Chr(sync_ein(n))
               Z = N - 1                          ' sonst geht's nicht
               Temp = Val(sync_ein(n))
               Writeeeprom Temp , Z
            Next N
         Case 48                                  ' = asc("0")
            Sync_aktiv = 0
            Twi_st_buff(2) = Sync_aktiv           ' vorsichtshalber
         Case 49                                  ' = asc("1")
            Sync_aktiv = 1                        ' Sync-Auswertung EIN
            Sync_event = 1                        ' Zwangsabfrage nach Einschalten der Sync-Auswertung
            Sync_neu = 1                          ' AVMatrix immer setzen
            Twi_st_buff(2) = Sync_aktiv
         Case 84                                  ' = asc("T")  (zum LM75 auslesen)
            Temp_aktiv = 1                        '
         Case 76                                  ' = asc("L")  Liste ausgeben
            Osdtext = "SYNC:" + Str(sync_aktiv)   '+ "   Timer:" + Str(timerbetrieb) 'Gibt's nicht mehr
            Osd_aktiv = 1 : Osd_event = 1
         End Select
      Case &HA8                                   ' .....es ist was abgeholt worden
         nop                                      ' "hier" wurden die Daten aus Twi_st_buff(n)abgeholt
      Case Else                                   ' irgendein Fehler. Was tun ???
         nop
   End Select                                     ' z.B Twi_mst_flag

   Twi_slv_flag = 0                               ' quittieren
   Twi_slv_stat = 0                               ' dto.
   Gosub Twi_sense                                ' Slave wieder scharfmachen
Return

'---------------------------------- Audio und Video Matrix ---------------------------------
Avmatrix:
   Select Case Syncmuster_14                      ' RX1 und RX2 Auswertung
   Debug "Avmatrix Case=" ; Syncmuster_14
      Case &B0000_0000
         Call Vmux(sync_ein(1) , 1) : Waitms 100
         Call Vmux(sync_ein(2) , 2) : Waitms 100
         Call Vmux(sync_ein(3) , 3) : Waitms 100
         Call Vmux(sync_ein(4) , 4) : Waitms 100
         Osdtext = ""                             ' Keine Sync-Einblendung
      Case &B1000_0000                            ' Sync von RX1
         Call Vmux(sync_ein(5) , 1) : Waitms 100  '
         Call Vmux(sync_ein(6) , 2) : Waitms 100
         Call Vmux(sync_ein(7) , 3) : Waitms 100
         Call Vmux(sync_ein(8) , 4) : Waitms 100
         Osdtext = "RX 1"
      Case &B0001_0000                            ' Sync von RX4
         Call Vmux(sync_ein(9) , 1) : Waitms 100
         Call Vmux(sync_ein(10) , 2) : Waitms 100
         Call Vmux(sync_ein(11) , 3) : Waitms 100
         Call Vmux(sync_ein(12) , 4) : Waitms 100
         Osdtext = "RX 4"
      Case &B1001_0000                            ' Sync von RX1 und RX4
         Call Vmux(sync_ein(13) , 1) : Waitms 100 '
         Call Vmux(sync_ein(14) , 2) : Waitms 100
         Call Vmux(sync_ein(15) , 3) : Waitms 100
         Call Vmux(sync_ein(16) , 4) : Waitms 100
         Osdtext = "RX 1 + RX 4"
   End Select
   Syncmuster_14_old = Syncmuster_14              ' Muster merken
'*******************************************************************************************
   If Osdtext <> "" Then                          '
      Osdstring = "*" + Osdtext + "*"             ' mu hier sein, sonst von Position im Main loop abhngig ???!!!???
      Call Osd(1 , Osdstring , 4)                 ' Meldung mit Startbefehl fr RX1
      Call Osd(4 , Osdstring , 4)                 ' ... und RX4
      Osd_aktiv = 1                               ' Osd_event = 1               ' flag setzen
      Osd_dauer = 0                               ' jetzt Zeit hochzhlen
   End If

Return

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'+++++++++++++++++++++++ PCF8574 auslesen = MMaster RX ++++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Pcf_lesen:                                        ' Portbelegung vom PCF lesen, invertieren und ...
   Fehler_flag = 0                                ' Anfangswert
   Do                                             ' ... per Maske ins Syncmuster umwandeln
      If Fehler_flag = Schleifen Then Goto Raus1  ' nach n Versuchen aufgeben
      Twi_mst_addr = Pcf0_adr + 1                 ' Pcf0 Adresse + 1
      Twi_mr_cntr = 1                             ' wieviele Byte
      Twi_mr_data = Varptr(twi_pcf0_buff)         ' Empfangsbuffer
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   Pcf0port = Not Twi_pcf0_buff                   ' wg. Portzuordnung
   Twi_st_buff(1) = Pcf0port                      ' +++ fr MMSlave Write +++++
   Debug "Pcf_lesen Twi_st_buff(1):" ; Twi_st_buff(1)       '

   Syncmuster_14 = Pcf0port And Maske_14          ' RX1 und 4 werden ausgewertet
   If Syncmuster_14 <> Syncmuster_14_old Then
      Sync_neu = 1
   Else
      Sync_neu = 0
   End If
   Debug "Syncmuster_14=" ; Syncmuster_14
   Debug "Syncmuster_14_old=" ; Syncmuster_14_old
   Debug "Sync_neu=" ; Sync_neu
   Debug "Ende Pcf_lesen"
Raus1:
Return


'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'+++++++++++++++++++++++ LM75 sensor MM read only ++++++++++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Sub Lm75(byval Rtxnum As Byte , Byval Lm75num As Byte , Byval Tnum As Byte)       '
   Dim Offset As Byte

   ''----------------- FRAM Bus ffnen ----------------
   If Rtxnum <> 0 Then                            ' falls FRAM Bus angesprochen
      Pcaschalter = Rtxnum - 1
      Pcaschalter = 2 ^ Pcaschalter               ' = 2^(Rtxnum-1)

      Twi_osd_buff(1) = Pcaschalter               ' ****** FRAM Bus ffnen *******
      Fehler_flag = 0
      Do
         If Fehler_flag = Schleifen Then Exit Sub ' nach n Versuchen abbrechen
         If Lm75num = 0 Then
            Twi_mst_addr = Pcatx                  ' TX Adresse (224)
         Else
            Twi_mst_addr = Pcarx                  ' RX Adresse (226)
         End If
         Twi_mt_cntr = 1                          ' 1 Byte senden
         Twi_mt_data = Varptr(twi_osd_buff(1))    ' Adresse Daten
         Twi_mr_cntr = 0
         Gosub Run_wait_master                    ' auf gehts
      Loop Until Twi_mst_flag = 0
   End If
   ''---------------------- LM75 Abfragen ----------------
   Offset = Lm75num                               ' in 2er Schritten
   Offset = Offset * 2
   Fehler_flag = 0                                ' von vorne zhlen
   Do
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Lm75_slave_addr + Offset     ' LM75 slave read Adresse + offset
      Twi_mr_cntr = 2                             ' LM75_high und LM75_low
      Twi_mr_data = Varptr(lm75_slave(1))         ' hier landen die Daten vom LM75slave
      Gosub Run_wait_master
   Loop Until Twi_mst_flag = 0
   Temperatur(tnum) = Lm75_slave(1)               '
   ''--------------- FRAM Bus schlieen -------------------
   If Rtxnum <> 0 Then
      Twi_osd_buff(1) = 0                         ' ****** FRAM Bus schlieen *******
      Fehler_flag = 0
      Do
         If Fehler_flag = Schleifen Then Exit Do  ' nach n Versuchen abbrechen
         If Lm75num = 0 Then
            Twi_mst_addr = Pcatx                  ' TX Adresse (224)
         Else
            Twi_mst_addr = Pcarx                  ' RX Adresse (226)
         End If
         Twi_mt_cntr = 1                          ' 1 Byte senden
         Twi_mt_data = Varptr(twi_osd_buff(1))    ' Adresse Daten
         Twi_mr_cntr = 0
         Gosub Run_wait_master                    ' auf gehts
      Loop Until Twi_mst_flag = 0
   End If
End Sub

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'++++++++++++++++++++++++  Text an OSD = MMaster TX ++++++++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Sub Osd(byval Rxnum As Byte , Byval Osdstring As String , Byval Befehl As Byte)
   Dim Temp1 As String * 1

   Pcaschalter = Rxnum - 1
   Pcaschalter = 2 ^ Pcaschalter                  ' = 2^(Rxnum-1)

   ''---------------------------------------------------------------
   Twi_osd_buff(1) = Pcaschalter                  ' ****** FRAM Bus ffnen *******
   Fehler_flag = 0
   Do
      If Fehler_flag = Schleifen Then Exit Sub    ' nach n Versuchen abbrechen
      Twi_mst_addr = Pcarx                        ' RX Adresse (224)
      Twi_mt_cntr = 1                             ' 1 Byte senden
      Twi_mt_data = Varptr(twi_osd_buff(1))       ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   ''-----------------------------------------------------------------
   If Len(osdstring) > 0 Then                     ' ****** Text + Befehl senden********
      For N = 1 To Len(osdstring)                 ' falls Text gegeben, als ASCII in array
         Temp1 = Mid(osdstring , N , 1)
         Twi_osd_buff(n) = Temp1
      Next N
      Twi_osd_buff(n) = Befehl                    ' OSD-Befehl ( z.B. Meldung EIN)
   Else
      Twi_osd_buff(1) = Befehl                    ' ... nur Befehl senden
   End If

   Fehler_flag = 0
   Do
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Osd_adr                      ' Adresse (96)
      Twi_mt_cntr = Len(osdstring) + 1            ' Text + Befehl
      Twi_mt_data = Varptr(twi_osd_buff(1))       ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   ''------------------------------------------------------------------
   Twi_osd_buff(1) = 0                            ' ******* FRAM Bus schlieen *********
   Fehler_flag = 0                                ' von vorne zhlen
   Do                                             '
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Pcarx                        ' Adresse (224)
      Twi_mt_cntr = 1                             ' 1 Byte senden
      Twi_mt_data = Varptr(twi_osd_buff(1))       ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   ''------------------------------------------------------------------
End Sub

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'++++++++++++++++++++++++++++++++ TX PTT setzen = MMaster TX +++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Sub Txset(byval Txnum As Byte , Byval Modus As Byte , Byval Schalter As Byte)       '

   Pcaschalter = Txnum - 1
   Pcaschalter = 2 ^ Pcaschalter                  ' = 2^(Txnum-1)            '
   '-----------------------------------------------------------------
   Twi_txset_buff(1) = Pcaschalter                ' *** FRAM "ffnen" ***
   Fehler_flag = 0                                ' von vorne zhlen
   Do                                             '
      If Fehler_flag = Schleifen Then Exit Sub    ' nach n Versuchen abbrechen
      Twi_mst_addr = Pcatx                        ' TX Adresse (226)
      Twi_mt_cntr = 1                             ' 1 Byte senden
      Twi_mt_data = Varptr(twi_txset_buff(1))     ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   '-----------------------------------------------------------------
   Twi_txset_buff(1) = Framtx_ptt                 ' Adresse (100+4)

   Twi_txset_buff(2) = Schalter                   ' =  Txset ein/aus    *** EIGENTLICHER BEFEHL ***
   Fehler_flag = 0
   Do
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Framaddr_tx                  ' Adresse (160)
      Twi_mt_cntr = 2                             ' 2 Byte senden
      Twi_mt_data = Varptr(twi_txset_buff(1))     ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   ''------------------------------------------------------------------
   Twi_txset_buff(1) = 0                          ' *** FRAM "schlieen" ***
   Fehler_flag = 0                                ' von vorne zhlen
   Do                                             '
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Pcatx                        ' Adresse (226)
      Twi_mt_cntr = 1                             ' 1 Byte senden
      Twi_mt_data = Varptr(twi_txset_buff(1))     ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   ''------------------------------------------------------------------

End Sub

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'++++++++++++++++++++++++++++ VMatrix setzen = MMaster TX ++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Sub Vmux(byval Vein As Byte , Byval Vaus As Byte) ' , Byval Schalter As Byte)
   Debug "Sub Vmux"
   If Vein = 0 Then Exit Sub                      ' keine nderung wenn Vein = 0 (aus Syncstandard)
   Twi_vmx_buff(1) = Vein                         '
   Twi_vmx_buff(2) = Vaus                         '
   Twi_vmx_buff(3) = 1                            ' Schalter (immer 1)
   Fehler_flag = 0                                ' von vorne zhlen
   Do                                             ' Vein und Vaus schreiben
      If Fehler_flag = Schleifen Then Exit Do     ' nach n Versuchen abbrechen
      Twi_mst_addr = Vmx_adr                      ' Adresse Vmux
      Twi_mt_cntr = 3                             ' 3 Byte senden
      Twi_mt_data = Varptr(twi_vmx_buff(1))       ' Adresse Daten
      Twi_mr_cntr = 0
      Gosub Run_wait_master                       ' auf gehts
   Loop Until Twi_mst_flag = 0
   Waitms 50
   Debug "Sub Vmux Ende"
   Debug " "                                      '
End Sub

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'+++++++++++++++++++++++++++ MultiMaster RX/TX Routine +++++++++++++++++++++++++++
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Run_wait_master:
   Gosub Mst_start                                ' Aufruf ( da drin passierts!!)
   Select Case Twi_mst_flag                       ' Ergebnis ?
      Case Twi_m_sent:                            ' (1) gesendet
         Twi_mst_flag = 0
      Case Twi_m_recv:                            ' (2) geholt
         Twi_mst_flag = 0
      Case Else                                   ' irgendein Problem z.B. (3)=timeout
         Incr Fehler_flag                         ' Flag setzen
         Twcr = &B00000000                        ' NNNNNNNNNNNNNNNNNNNNNNNNNNN
         Twcr = &B00000100                        ' NNNNNNNNNNNNNNNNNNNNNNNNNNN
         Gosub Twi_sense                          ' NNNNNNNNNNNNNNNNNNNNNNNNNNN
   End Select
   Twi_st_buff(5) = Twi_mst_flag                  ' Fehlerauswertung
   Debug "Twi_mst_flag=" ; Twi_mst_flag
Return
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++